@@ -1,17 +1,20 @@
 # plot method for rectangular smacof
 
-plot.smacofR <- function(x, plot.type = "confplot", joint = TRUE, plot.dim = c(1,2), 
+plot.smacofR <- function(x, plot.type = "confplot", what = c("both", "columns", "rows"), plot.dim = c(1,2), 
                          col.rows = hcl(0), 
                          col.columns = hcl(240),
                          label.conf.rows = list(label = TRUE, pos = 3, col = hcl(0, l = 50), cex = 0.8), 
                          label.conf.columns = list(label = TRUE, pos = 3, col = hcl(240, l = 50), cex = 0.8), 
-                         type = "p", pch = 20, asp = 1, main, xlab, ylab, xlim, ylim, ...)
+                         type = "p", pch = 20, cex = 0.5, asp = 1, main, xlab, ylab, xlim, ylim, ...)
 
 # x ... object of class smacofR
 # plot.type ... types available: "confplot", "Shepard", "stressplot", "resplot"
 # joint ... if TRUE, row and column configurations in 1 plot
   
 {
+  plot.type <- match.arg(plot.type, c("confplot", "stressplot", "Shepard"), several.ok = FALSE)
+  what <- match.arg(what, c("both", "columns", "rows"), several.ok = FALSE)
+  
   x1 <- plot.dim[1]
   y1 <- plot.dim[2]
   
@@ -20,6 +23,7 @@ plot.smacofR <- function(x, plot.type = "confplot", joint = TRUE, plot.dim = c(1
   if (is.null(label.conf.rows$pos)) label.conf.rows$pos <- 3
   if (is.null(label.conf.rows$col)) label.conf.rows$col <- hcl(0, l = 50)
   if (is.null(label.conf.rows$cex)) label.conf.rows$cex <- 0.8
+ 
   if (is.null(label.conf.columns$label)) label.conf.columns$label <- TRUE
   if (is.null(label.conf.columns$pos)) label.conf.columns$pos <- 3
   if (is.null(label.conf.columns$col)) label.conf.columns$col <- hcl(240, l = 50)
@@ -31,36 +35,39 @@ plot.smacofR <- function(x, plot.type = "confplot", joint = TRUE, plot.dim = c(1
     #if (missing(type)) type <- "n" else type <- type
     if (missing(xlab)) xlab1 <- paste("Dimension", x1,sep = " ") else xlab1 <- xlab  
     if (missing(ylab)) ylab1 <- paste("Dimension", y1,sep = " ") else ylab1 <- ylab
-    ppos.rows <- label.conf.rows[[2]]
-    ppos.columns <- label.conf.columns[[2]]
+    ppos.rows <- label.conf.rows$pos
+    ppos.columns <- label.conf.columns$pos
     if (type == "n") ppos.rows <- ppos.columns <- NULL
 
-    if (!joint) {                                                                                 
-      op <- par(mfrow = c(1,2))
+    if (what == "columns") {                                                                                 
       if (missing(main)) main1 <- paste("Configuration Plot - Columns") else main1 <- main        #plot column configurations
       #if (missing(xlab)) xlab1 <- paste("Column Configurations D", x1,sep = "") else xlab1 <- xlab
       #if (missing(xlab)) ylab1 <- paste("Column Configurations D", y1,sep = "") else ylab1 <- ylab
 
       if (missing(xlim)) xlim <- range(x$conf.col[,c(x1,y1)])
-      if (missing(ylim)) ylim <- range(x$conf.row[,c(x1,y1)])
-
+      if (missing(ylim)) ylim <- range(x$conf.col[,c(x1,y1)])
+      
       plot(x$conf.col[,x1], x$conf.col[,y1], main = main1, xlab = xlab1, ylab = ylab1, type = type, 
-           xlim = xlim, ylim = ylim, col = col.columns, pch = pch, asp = asp, ...)
-      if (label.conf.columns[[1]]) text(x$conf.col[,x1], x$conf.col[,y1], labels = rownames(x$conf.col), 
+           xlim = xlim, ylim = ylim, col = col.columns, pch = pch, asp = asp, cex = cex, ...)
+      if (label.conf.columns$label) text(x$conf.col[,x1], x$conf.col[,y1], labels = rownames(x$conf.col), 
                                         cex = label.conf.columns$cex, pos = ppos.columns, 
-                                        col = label.conf.columns[[3]])
-
+                                        col = label.conf.columns$col)
+    }
+    if (what == "rows") {                                                                                 
       if (missing(main)) main1 <- paste("Configuration Plot - Rows") else main1 <- main       #plot row configurations
       #if (missing(xlab)) xlab1 <- paste("Column Configurations D", x1,sep = "") else xlab1 <- xlab
       #if (missing(ylab)) ylab1 <- paste("Column Configurations D", y1,sep = "") else ylab1 <- ylab
       
+      if (missing(xlim)) xlim <- range(x$conf.row[,c(x1,y1)])
+      if (missing(ylim)) ylim <- range(x$conf.row[,c(x1,y1)])
+      
       plot(x$conf.row[,x1], x$conf.row[,y1], main = main1, xlab = xlab1, ylab = ylab1, type = type, 
-           xlim = xlim, ylim = ylim, col = col.rows, pch = pch, asp = asp, ...)
-      if (label.conf.rows[[1]]) text(x$conf.row[,x1], x$conf.row[,y1], labels = rownames(x$conf.row), 
-                                     cex = label.conf.rows$cex, pos = ppos.rows, col = label.conf.rows[[3]])      
-      par(op)
+           xlim = xlim, ylim = ylim, col = col.rows, pch = pch, asp = asp, cex = cex, ...)
+      if (label.conf.rows$label) text(x$conf.row[,x1], x$conf.row[,y1], labels = rownames(x$conf.row), 
+                                     cex = label.conf.rows$cex, pos = ppos.rows, col = label.conf.rows$col)      
+    }
     
-    } else { #joint plot
+    if (what == "both") {  #joint plot
       if (missing(main)) main1 <- paste("Joint Configuration Plot") else main1 <- main        
       
       fullconf <- rbind(x$conf.col[,c(x1,y1)],x$conf.row[,c(x1,y1)])
@@ -68,37 +75,38 @@ plot.smacofR <- function(x, plot.type = "confplot", joint = TRUE, plot.dim = c(1
       if (missing(ylim)) ylim <- range(fullconf)
       
       plot(x$conf.row[,x1], x$conf.row[,y1], main = main1, xlab = xlab1, ylab = ylab1, type = type, 
-             xlim = xlim, ylim = ylim, col = col.rows, pch = pch, asp = asp,...)
-      if (label.conf.rows[[1]]) text(x$conf.row[,x1], x$conf.row[,y1], labels = rownames(x$conf.row),
-                                     cex = label.conf.rows$cex, pos = ppos.rows, col = label.conf.rows[[3]])    
+             xlim = xlim, ylim = ylim, col = col.rows, pch = pch, asp = asp, cex = cex, ...)
+      if (label.conf.rows$label) text(x$conf.row[,x1], x$conf.row[,y1], labels = rownames(x$conf.row),
+                                     cex = label.conf.rows$cex, pos = ppos.rows, col = label.conf.rows$col)    
       points(x$conf.col[,x1], x$conf.col[,y1], main = main1, xlab = xlab1, ylab = ylab1, type = type, 
-           xlim = xlim, ylim = ylim, col = col.columns, pch = pch, ...)
-      if (label.conf.columns[[1]]) text(x$conf.col[,x1], x$conf.col[,y1], labels = rownames(x$conf.col), 
+           xlim = xlim, ylim = ylim, col = col.columns, pch = pch, cex = cex, ...)
+      if (label.conf.columns$label) text(x$conf.col[,x1], x$conf.col[,y1], labels = rownames(x$conf.col), 
                                         cex = label.conf.columns$cex, pos = ppos.columns, 
-                                        col = label.conf.columns[[3]])
+                                        col = label.conf.columns$col)
     }
   }
 
   #---------------- Shepard diagram ------------------
-#   if (plot.type == "Shepard") {
-#     if (missing(main)) main <- paste("Shepard Diagram") else main <- main
-#     if (missing(xlab)) xlab <- "Dissimilarities" else xlab <- xlab
-#     if (missing(ylab)) ylab <- "Configuration Distances" else ylab <- ylab
-# 
-#     if (missing(xlim)) xlim <- range(as.vector(x$obsdiss))
-#     if (missing(ylim)) ylim <- range(as.vector(x$confdiss))
-# 
-#     plot(as.vector(x$obsdiss), as.vector(x$confdiss), main = main, type = "p", pch = 1,
-#          xlab = xlab, ylab = ylab, col = "darkgray", xlim = xlim, ylim = ylim, ...)
-# 
-#     if (!x$metric) {
-#       isofit <- isoreg(as.vector(x$obsdiss), as.vector(x$confdiss))  #isotonic regression
-#       points(sort(isofit$x), isofit$yf, type = "b", pch = 16)
-#     } else {
-#       abline(0,1, lty = 2)
-#     }
-#    
-#   }
+  if (plot.type == "Shepard") {
+    if (missing(main)) main <- paste("Shepard Diagram") else main <- main
+    if (missing(xlab)) xlab <- "Dissimilarities" else xlab <- xlab
+    if (missing(ylab)) ylab <- "Configuration Distances" else ylab <- ylab
+
+    if (missing(xlim)) xlim <- range(as.vector(x$obsdiss))
+    if (missing(ylim)) ylim <- range(as.vector(x$confdist))
+    
+    notmiss <- as.vector(x$weightmat > 0)
+    xcoor <- as.vector(x$obsdiss)
+    ycoor <- as.vector(x$confdist)
+    plot(xcoor[notmiss], ycoor[notmiss], main = main, type = "p", pch = pch, cex = cex,
+         xlab = xlab, ylab = ylab, col = "darkgray", xlim = xlim, ylim = ylim, ...)
+    yax <- ycoor[notmiss]
+    xax <- xcoor[notmiss]
+    fitlm <- lm(yax ~ -1 + xax)       ## ratio unfolding
+    xvals <- unique(sort(xax))
+    preds <- predict(fitlm, newdata = data.frame(xax = xvals))
+    points(xvals, preds, type = "b", pch = pch, cex = cex)
+}
 
 
   #--------------- Residual plot --------------------
@@ -150,6 +158,6 @@ plot.smacofR <- function(x, plot.type = "confplot", joint = TRUE, plot.dim = c(1
          xlab = xlab2, ylab = ylab, main = main2, xlim = xlim1, ylim = ylim1, ...)
     text(1:length(spp.perc.col), spp.perc.col, labels = xaxlab, pos = 3, cex = 0.8)
     for (i in 1:length(spp.perc.col)) lines(c(i,i), c(spp.perc.col[i],0), col = "lightgray", lty = 2)  
-    par(op)  
+    on.exit(par(op))  
   }
 }
